'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.startDevServer = startDevServer;

var _plugins = require('./plugins');

var _util = require('./util');

var _Server = require('webpack-dev-server/lib/Server');

var _Server2 = _interopRequireDefault(_Server);

var _createLog = require('webpack-dev-server/lib/createLog');

var _createLog2 = _interopRequireDefault(_createLog);

var _PromiseDefer = require('./PromiseDefer');

var _PromiseDefer2 = _interopRequireDefault(_PromiseDefer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function colorInfo(msg) {
  return '\x1B[1m\x1B[34m' + msg + '\x1B[39m\x1B[22m';
} /**************************************************
   * Created by nanyuantingfeng on 23/08/2017 14:29.
   **************************************************/


function colorError(msg) {
  return '\x1B[1m\x1B[31m' + msg + '\x1B[39m\x1B[22m';
}

function startDevServer(context) {
  var webpackConfig = context.webpackConfig;


  var firstWpOpt = Array.isArray(webpackConfig) ? webpackConfig[0] : webpackConfig;

  var options = webpackConfig.devServer || firstWpOpt.devServer || {};

  var log = (0, _createLog2.default)(options);

  _Server2.default.addDevServerEntrypoints(webpackConfig, options);

  var defer = (0, _PromiseDefer2.default)();

  var compiler = void 0;

  try {

    compiler = (0, _plugins.webpack)(webpackConfig);
  } catch (e) {

    if (e instanceof _plugins.WebpackOptionsValidationError) {
      log.error(colorError(e.message));
      process.exit(1);
    }

    defer.reject(e);
  }

  var server = void 0;

  try {
    server = new _Server2.default(compiler, options, log);
  } catch (e) {
    var OptionsValidationError = require('webpack-dev-server/lib/OptionsValidationError');

    if (e instanceof OptionsValidationError) {
      log.error(colorError(e.message));
      process.exit(1);
    }

    defer.reject(e);
  }

  ['SIGINT', 'SIGTERM'].forEach(function (sig) {
    process.on(sig, function () {
      server.close(function () {
        process.exit();
      });
    });
  });

  server.listen(options.port, options.host, function (err) {
    if (err) throw err;
    defer.resolve(server);
    log.info('\nService is running at ' + colorInfo((0, _util.createDomain)(options)));
  });

  return defer.promise;
}